import 'dart:math';

import 'package:flutter/foundation.dart';

import 'vector3.dart';

@immutable
class FVector2 {
  const FVector2(this.x, this.y);

  final double x;
  final double y;

  double get norm => abs();

  factory FVector2.zeros() => FVector2(0, 0);
  factory FVector2.ones() => FVector2(1, 1);
  factory FVector2.fromVector3XY(FVector3 v3) => FVector2(v3.x, v3.y);
  factory FVector2.fromVector3XZ(FVector3 v3) => FVector2(v3.x, v3.z);
  factory FVector2.fromVector3YZ(FVector3 v3) => FVector2(v3.y, v3.z);

  FVector2 copyWith({double? x, double? y}) {
    return FVector2(x ?? this.x, y ?? this.y);
  }

  double abs() {
    return sqrt(dot(this));
  }

  FVector2 normed() {
    return this / norm;
  }

  double dot(FVector2 v) {
    return x * v.x + y * v.y;
  }

  FVector2 to(FVector2 v) {
    return v - this;
  }

  double distanceTo(FVector2 v) {
    return (v - this).norm;
  }

  FVector2 operator +(FVector2 v) {
    return FVector2(x + v.x, y + v.y);
  }

  FVector2 operator -(FVector2 v) {
    return FVector2(x - v.x, y - v.y);
  }

  FVector2 operator *(double d) {
    return FVector2(d * x, d * y);
  }

  FVector2 operator /(double d) {
    return FVector2(x / d, y / d);
  }
}
