import 'dart:math';

import 'package:flutter/foundation.dart';

@immutable
class FVector3 {
  const FVector3(this.x, this.y, this.z);

  final double x;
  final double y;
  final double z;

  double get norm => abs();

  factory FVector3.zeros() => FVector3(0, 0, 0);
  factory FVector3.ones() => FVector3(1, 1, 1);
  factory FVector3.infinite() =>
      FVector3(double.infinity, double.infinity, double.infinity);

  FVector3 copyWith({double? x, double? y, double? z}) {
    return FVector3(x ?? this.x, y ?? this.y, z ?? this.z);
  }

  double abs() {
    return sqrt(dot(this));
  }

  FVector3 normed() {
    return this / norm;
  }

  double dot(FVector3 v) {
    return x * v.x + y * v.y + z * v.z;
  }

  FVector3 cross(FVector3 v) {
    return FVector3(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
  }

  FVector3 to(FVector3 v) {
    return v - this;
  }

  double distanceTo(FVector3 v) {
    return (v - this).norm;
  }

  FVector3 operator +(FVector3 v) {
    return FVector3(x + v.x, y + v.y, z + v.z);
  }

  FVector3 operator -(FVector3 v) {
    return FVector3(x - v.x, y - v.y, z - v.z);
  }

  FVector3 operator *(double d) {
    return FVector3(d * x, d * y, d * z);
  }

  FVector3 operator /(double d) {
    return FVector3(x / d, y / d, z / d);
  }
}
